// Microsoft Visual C++ generated resource script.
//
#include "resource.h"

#define APSTUDIO_READONLY_SYMBOLS
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//
#include "winres.h"

/////////////////////////////////////////////////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
// German (Germany) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_DEU)
LANGUAGE LANG_GERMAN, SUBLANG_GERMAN

#ifdef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE 
BEGIN
    "resource.h\0"
END

2 TEXTINCLUDE 
BEGIN
    "#include ""winres.h""\r\n"
    "\0"
END

3 TEXTINCLUDE 
BEGIN
    "\r\n"
    "\0"
END

#endif    // APSTUDIO_INVOKED

#endif    // German (Germany) resources
/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
// English (United States) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENU)
LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_US

/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

IDD_DIALOG1 DIALOGEX 0, 0, 171, 212
STYLE DS_SETFONT | DS_MODALFRAME | DS_FIXEDSYS | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "Dialog"
FONT 8, "MS Shell Dlg", 400, 0, 0x1
BEGIN
    GROUPBOX        "General",IDC_STATIC,6,6,156,78
    CONTROL         "Multithreaded rendering",IDC_CHECK_MULTITHREAD,"Button",BS_AUTOCHECKBOX | WS_TABSTOP,12,18,93,10
    RTEXT           "Threads:",IDC_STATIC,12,36,48,8
    EDITTEXT        IDC_EDIT_WORKERS,66,36,36,14,ES_AUTOHSCROLL | ES_NUMBER
    CONTROL         "",IDC_SPIN_WORKERS,"msctls_updown32",UDS_SETBUDDYINT | UDS_ALIGNRIGHT | UDS_AUTOBUDDY | UDS_ARROWKEYS,90,36,11,14
    LTEXT           "(0 = auto)",IDC_STATIC,108,39,34,8
    GROUPBOX        "Video Interface",IDC_STATIC,6,90,156,96
    RTEXT           "Output mode:",IDC_STATIC,12,103,48,8
    COMBOBOX        IDC_COMBO_VI_MODE,66,102,90,72,CBS_DROPDOWNLIST | WS_VSCROLL | WS_TABSTOP
    CONTROL         "16:9 widescreen",IDC_CHECK_VI_WIDESCREEN,"Button",BS_AUTOCHECKBOX | WS_TABSTOP,12,144,66,10
    CONTROL         "Hide overscan",IDC_CHECK_VI_OVERSCAN,"Button",BS_AUTOCHECKBOX | WS_TABSTOP,96,144,60,10
    DEFPUSHBUTTON   "OK",IDOK,6,192,50,14
    PUSHBUTTON      "Cancel",IDCANCEL,60,192,50,14
    PUSHBUTTON      "Apply",IDAPPLY,114,192,50,14
    COMBOBOX        IDC_COMBO_VI_INTERP,66,126,90,30,CBS_DROPDOWNLIST | WS_VSCROLL | WS_TABSTOP
    RTEXT           "Interpolation:",IDC_STATIC,12,126,48,8
    CONTROL         "Enable VSync",IDC_CHECK_VI_VSYNC,"Button",BS_AUTOCHECKBOX | WS_TABSTOP,96,156,59,10
    CONTROL         "Exclusive fullscreen",IDC_CHECK_VI_EXCLUSIVE,"Button",BS_AUTOCHECKBOX | WS_TABSTOP,12,156,78,10
    COMBOBOX        IDC_COMBO_DP_COMPAT,66,60,90,30,CBS_DROPDOWNLIST | WS_VSCROLL | WS_TABSTOP
    RTEXT           "Compatibility:",IDC_STATIC,12,60,48,8
    CONTROL         "Integer pixel scaling",IDC_CHECK_VI_INTEGER_SCALING,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,12,168,80,10
    CONTROL         "Enable busyloop",IDC_CHECK_BUSYLOOP,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,96,168,80,10
END


/////////////////////////////////////////////////////////////////////////////
//
// DESIGNINFO
//

#ifdef APSTUDIO_INVOKED
GUIDELINES DESIGNINFO
BEGIN
    IDD_DIALOG1, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 164
        TOPMARGIN, 7
        BOTTOMMARGIN, 205
    END
END
#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// AFX_DIALOG_LAYOUT
//

IDD_DIALOG1 AFX_DIALOG_LAYOUT
BEGIN
    0
END

#endif    // English (United States) resources
/////////////////////////////////////////////////////////////////////////////



#ifndef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//


/////////////////////////////////////////////////////////////////////////////
#endif    // not APSTUDIO_INVOKED

