//{{NO_DEPENDENCIES}}
// Microsoft Visual C++ generated include file.
// Used by config.rc
//
#define IDD_DIALOG1                     101
#define IDC_COMBO_VI_MODE               1003
#define IDC_EDIT_WORKERS                1004
#define IDC_SPIN_WORKERS                1006
#define IDC_CHECK_VI_WIDESCREEN         1007
#define IDC_CHECK_VI_OVERSCAN           1008
#define IDC_CHECK_MULTITHREAD           1009
#define IDAPPLY                         1010
#define IDC_COMBO_VI_INTERP             1011
#define IDC_CHECK_VI_VSYNC              1012
#define IDC_CHECK_VI_EXCLUSIVE          1013
#define IDC_COMBO_DP_COMPAT             1014
#define IDC_CHECK_VI_EXCLUSIVE2         1015
#define IDC_CHECK_VI_INTEGER_SCALING    1015
#define IDC_CHECK_BUSYLOOP              1016

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        103
#define _APS_NEXT_COMMAND_VALUE         40001
#define _APS_NEXT_CONTROL_VALUE         1015
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
